==> nestjs-microservices-cqrs-1.0.4.tgz: package/src/services/service.ts <==
import { Injectable } from '@nestjs/common';
import {
    ClientKafka,
    ClientOptions,
    ClientProxyFactory,
    CustomStrategy,
    MessageHandler,
    MicroserviceOptions, ReadPacket,
    Server, WritePacket
} from '@nestjs/microservices';
import {ServerFactory} from "@nestjs/microservices/server/server-factory";
import {ClientProxy} from "@nestjs/microservices/client/client-proxy";
import {IEvent} from "@nestjs/cqrs";
import {Observable, from, of} from "rxjs";

interface IMessageHandlerGlobal extends MessageHandler {
    isGlobal?: boolean;
}

abstract class SubscribableClientProxy extends ClientProxy {
    public abstract connect(): Promise<any>;

    public abstract close(): any;

    protected abstract publish(
        packet: ReadPacket,
        callback: (packet: WritePacket) => void,
    ): Function;

    protected abstract dispatchEvent<T = any>(packet: ReadPacket): Promise<T>;

    abstract subscribeToResponseOf(pattern: any): void;
}

@Injectable()
export class NestjsMicroservicesCqrs<EventBase extends IEvent = IEvent> {
    private options: MicroserviceOptions | undefined;
    private server: Server | undefined;
    private client: SubscribableClientProxy | undefined;

    run(messagePatterns: string[], options: MicroserviceOptions) {
        if (!this.client) {
            this._createClient(messagePatterns, options);
        }
    }

    init( options: MicroserviceOptions ) {
        return this._createServer(options);
    };

    private _createServer(options: MicroserviceOptions) {
        this.options = options;
        const server = ServerFactory.create(options);
        this.server = server;
        this.client = undefined;
        return { strategy: server } as CustomStrategy;
    }

    private _createClient(messagePatterns: string[], options: MicroserviceOptions) {
        const client = <SubscribableClientProxy>ClientProxyFactory.create(<ClientOptions>options);
        if (client) {
            if (typeof client.subscribeToResponseOf === 'function') {
                messagePatterns.forEach(message => client.subscribeToResponseOf(message));
            }
            client.connect();
            this.client = client
        }
    }

    addHandler<U>(pattern: string, handler: (name: string, data: any) => U) {
        if (this.server) {
            let oldHandler: IMessageHandlerGlobal | null;
            oldHandler = this.server.getHandlerByPattern(pattern);
            let finalHandler: IMessageHandlerGlobal;
            if (!(oldHandler && oldHandler.isGlobal)) {
                // @ts-ignore
                finalHandler = (data:any, ctx?:any) => {
                    let res: any;
                    if (oldHandler instanceof Function) {
                        res = oldHandler(data, ctx);
                    }
                    const res2 = handler(pattern, data.value);
                    return Promise.resolve(res2);
                };
                finalHandler.isEventHandler = true;
                finalHandler.isGlobal = true;
                this.server.addHandler(pattern, finalHandler, true);
            }
        }
    }

    emit<TResult = any>(pattern: string, data: any): Promise<Observable<TResult>> | null {
        if (this.client) {
            return this.client.emit(pattern, data).toPromise();
        }
        return null;
    }

    send<TResult = any>(pattern: string, data: any): Observable<TResult> {
        if (this.client) {
            return this.client.send(pattern, data);
        }
        return of(null as unknown as TResult);
    }
}

==> nestjs-microservices-cqrs-1.0.5.tgz: package/src/services/service.ts <==
import { Injectable } from '@nestjs/common';
import {
    ClientKafka,
    ClientOptions,
    ClientProxyFactory,
    CustomStrategy,
    MessageHandler,
    MicroserviceOptions, ReadPacket,
    Server, WritePacket
} from '@nestjs/microservices';
import {ServerFactory} from "@nestjs/microservices/server/server-factory";
import {ClientProxy} from "@nestjs/microservices/client/client-proxy";
import {IEvent} from "@nestjs/cqrs";
import {Observable, from, of} from "rxjs";

interface IMessageHandlerGlobal extends MessageHandler {
    isGlobal?: boolean;
}

abstract class SubscribableClientProxy extends ClientProxy {
    public abstract connect(): Promise<any>;

    public abstract close(): any;

    protected abstract publish(
        packet: ReadPacket,
        callback: (packet: WritePacket) => void,
    ): Function;

    protected abstract dispatchEvent<T = any>(packet: ReadPacket): Promise<T>;

    abstract subscribeToResponseOf(pattern: any): void;
}

@Injectable()
export class NestjsMicroservicesCqrs<EventBase extends IEvent = IEvent> {
    private options: MicroserviceOptions | undefined;
    private server: Server | undefined;
    private fromBeginningServer: Server | undefined;
    private client: SubscribableClientProxy | undefined;

    run(messagePatterns: string[]) {
        if (!this.client) {
            this._createClient(messagePatterns);
        }
    }

    init( options: MicroserviceOptions, fromBeginningConfig?: MicroserviceOptions ) {
        return this._createServer(options, fromBeginningConfig || options);
    };

    private _createServer(options: MicroserviceOptions, fromBeginningConfig: MicroserviceOptions) {
        this.options = options;
        const server = ServerFactory.create(options);
        const fromBeginningServer = ServerFactory.create(fromBeginningConfig);
        const serverListen = server.listen;
        const tempListen = async (callback: any) => {
            await fromBeginningServer.listen(callback);
            if (this.client && this.client.connect) {
                await this.client.connect();
            }
            await serverListen.bind(server)(callback);
        };
        server.listen = tempListen;
        this.server = server;
        this.fromBeginningServer = fromBeginningServer;
        this.client = undefined;
        return { strategy: server } as CustomStrategy;
    }

    private _createClient(messagePatterns: string[]) {
        const client = <SubscribableClientProxy>ClientProxyFactory.create(<ClientOptions>this.options);
        if (client) {
            if (typeof client.subscribeToResponseOf === 'function') {
                messagePatterns.forEach(message => client.subscribeToResponseOf(message));
            }
            // client.connect();
            this.client = client
        }
    }

    addHandler<U>(pattern: string, handler: (name: string, data: any) => U) {
        if (this.fromBeginningServer) {
            let oldHandler: IMessageHandlerGlobal | null;
            oldHandler = this.fromBeginningServer.getHandlerByPattern(pattern);
            let finalHandler: IMessageHandlerGlobal;
            if (!(oldHandler && oldHandler.isGlobal)) {
                // @ts-ignore
                finalHandler = (data:any, ctx?:any) => {
                    let res: any;
                    if (oldHandler instanceof Function) {
                        res = oldHandler(data, ctx);
                    }
                    const res2 = handler(pattern, data.value);
                    return Promise.resolve(res2);
                };
                finalHandler.isEventHandler = true;
                finalHandler.isGlobal = true;
                this.fromBeginningServer.addHandler(pattern, finalHandler, true);
            }
        }
    }

    emit<TResult = any>(pattern: string, data: any): Promise<Observable<TResult>> | null {
        if (this.client) {
            return this.client.emit(pattern, data).toPromise();
        }
        return null;
    }

    send<TResult = any>(pattern: string, data: any): Observable<TResult> {
        if (this.client) {
            return this.client.send(pattern, data);
        }
        return of(null as unknown as TResult);
    }
}
